@@ -26,7 +26,7 @@ describe('WebSocket', () => {
       );
     });
 
-    it('accepts `url.URL` objects as url', function(done) {
+    it('accepts `url.URL` objects as url', (done) => {
       const agent = new CustomAgent();
 
       agent.addRequest = (req, opts) => {
@@ -100,21 +100,28 @@ describe('WebSocket', () => {
     Object.keys(readyStates).forEach((state) => {
       describe(`\`${state}\``, () => {
         it('is enumerable property of class', () => {
-          const propertyDescripter = Object.getOwnPropertyDescriptor(
-            WebSocket,
+          const descriptor = Object.getOwnPropertyDescriptor(WebSocket, state);
+
+          assert.deepStrictEqual(descriptor, {
+            configurable: false,
+            enumerable: true,
+            value: readyStates[state],
+            writable: false
+          });
+        });
+
+        it('is enumerable property of prototype', () => {
+          const descriptor = Object.getOwnPropertyDescriptor(
+            WebSocket.prototype,
             state
           );
 
-          assert.strictEqual(propertyDescripter.value, readyStates[state]);
-          assert.strictEqual(propertyDescripter.enumerable, true);
-        });
-
-        it('is property of instance', () => {
-          const ws = new WebSocket('ws://localhost', {
-            agent: new CustomAgent()
+          assert.deepStrictEqual(descriptor, {
+            configurable: false,
+            enumerable: true,
+            value: readyStates[state],
+            writable: false
           });
-
-          assert.strictEqual(ws[state], readyStates[state]);
         });
       });
     });
@@ -122,6 +129,18 @@ describe('WebSocket', () => {
 
   describe('Attributes', () => {
     describe('`binaryType`', () => {
+      it('is enumerable and configurable', () => {
+        const descriptor = Object.getOwnPropertyDescriptor(
+          WebSocket.prototype,
+          'binaryType'
+        );
+
+        assert.strictEqual(descriptor.configurable, true);
+        assert.strictEqual(descriptor.enumerable, true);
+        assert.ok(descriptor.get !== undefined);
+        assert.ok(descriptor.set !== undefined);
+      });
+
       it("defaults to 'nodebuffer'", () => {
         const ws = new WebSocket('ws://localhost', {
           agent: new CustomAgent()
@@ -153,6 +172,18 @@ describe('WebSocket', () => {
     });
 
     describe('`bufferedAmount`', () => {
+      it('is enumerable and configurable', () => {
+        const descriptor = Object.getOwnPropertyDescriptor(
+          WebSocket.prototype,
+          'bufferedAmount'
+        );
+
+        assert.strictEqual(descriptor.configurable, true);
+        assert.strictEqual(descriptor.enumerable, true);
+        assert.ok(descriptor.get !== undefined);
+        assert.ok(descriptor.set === undefined);
+      });
+
       it('defaults to zero', () => {
         const ws = new WebSocket('ws://localhost', {
           agent: new CustomAgent()
@@ -185,13 +216,16 @@ describe('WebSocket', () => {
 
             ws.on('open', () => {
               ws.send('foo');
+
+              assert.strictEqual(ws.bufferedAmount, 3);
+
               ws.send('bar', (err) => {
                 assert.ifError(err);
                 assert.strictEqual(ws.bufferedAmount, 0);
                 wss.close(done);
               });
 
-              assert.strictEqual(ws.bufferedAmount, 3);
+              assert.strictEqual(ws.bufferedAmount, 6);
             });
           }
         );
@@ -205,12 +239,15 @@ describe('WebSocket', () => {
         wss.on('connection', (ws) => {
           const data = Buffer.alloc(1024, 61);
 
-          while (ws._socket.bufferSize === 0) {
+          while (ws.bufferedAmount === 0) {
             ws.send(data);
           }
 
-          assert.ok(ws._socket.bufferSize > 0);
-          assert.strictEqual(ws.bufferedAmount, ws._socket.bufferSize);
+          assert.ok(ws.bufferedAmount > 0);
+          assert.strictEqual(
+            ws.bufferedAmount,
+            ws._socket._writableState.length
+          );
 
           ws.on('close', () => wss.close(done));
           ws.close();
@@ -219,6 +256,18 @@ describe('WebSocket', () => {
     });
 
     describe('`extensions`', () => {
+      it('is enumerable and configurable', () => {
+        const descriptor = Object.getOwnPropertyDescriptor(
+          WebSocket.prototype,
+          'bufferedAmount'
+        );
+
+        assert.strictEqual(descriptor.configurable, true);
+        assert.strictEqual(descriptor.enumerable, true);
+        assert.ok(descriptor.get !== undefined);
+        assert.ok(descriptor.set === undefined);
+      });
+
       it('exposes the negotiated extensions names (1/2)', (done) => {
         const wss = new WebSocket.Server({ port: 0 }, () => {
           const ws = new WebSocket(`ws://localhost:${wss.address().port}`);
@@ -263,6 +312,18 @@ describe('WebSocket', () => {
     });
 
     describe('`protocol`', () => {
+      it('is enumerable and configurable', () => {
+        const descriptor = Object.getOwnPropertyDescriptor(
+          WebSocket.prototype,
+          'protocol'
+        );
+
+        assert.strictEqual(descriptor.configurable, true);
+        assert.strictEqual(descriptor.enumerable, true);
+        assert.ok(descriptor.get !== undefined);
+        assert.ok(descriptor.set === undefined);
+      });
+
       it('exposes the subprotocol selected by the server', (done) => {
         const wss = new WebSocket.Server({ port: 0 }, () => {
           const port = wss.address().port;
@@ -284,6 +345,18 @@ describe('WebSocket', () => {
     });
 
     describe('`readyState`', () => {
+      it('is enumerable and configurable', () => {
+        const descriptor = Object.getOwnPropertyDescriptor(
+          WebSocket.prototype,
+          'readyState'
+        );
+
+        assert.strictEqual(descriptor.configurable, true);
+        assert.strictEqual(descriptor.enumerable, true);
+        assert.ok(descriptor.get !== undefined);
+        assert.ok(descriptor.set === undefined);
+      });
+
       it('defaults to `CONNECTING`', () => {
         const ws = new WebSocket('ws://localhost', {
           agent: new CustomAgent()
@@ -333,6 +406,18 @@ describe('WebSocket', () => {
     });
 
     describe('`url`', () => {
+      it('is enumerable and configurable', () => {
+        const descriptor = Object.getOwnPropertyDescriptor(
+          WebSocket.prototype,
+          'url'
+        );
+
+        assert.strictEqual(descriptor.configurable, true);
+        assert.strictEqual(descriptor.enumerable, true);
+        assert.ok(descriptor.get !== undefined);
+        assert.ok(descriptor.set === undefined);
+      });
+
       it('exposes the server url', () => {
         const url = 'ws://localhost';
         const ws = new WebSocket(url, { agent: new CustomAgent() });
@@ -1345,9 +1430,7 @@ describe('WebSocket', () => {
             'Invalid WebSocket frame: MASK must be set'
           );
           assert.ok(
-            Buffer.concat(chunks)
-              .slice(0, 2)
-              .equals(Buffer.from('8102', 'hex'))
+            Buffer.concat(chunks).slice(0, 2).equals(Buffer.from('8102', 'hex'))
           );
 
           ws.on('close', (code, reason) => {
@@ -1725,6 +1808,18 @@ describe('WebSocket', () => {
 
   describe('WHATWG API emulation', () => {
     it('supports the `on{close,error,message,open}` attributes', () => {
+      for (const property of ['onclose', 'onerror', 'onmessage', 'onopen']) {
+        const descriptor = Object.getOwnPropertyDescriptor(
+          WebSocket.prototype,
+          property
+        );
+
+        assert.strictEqual(descriptor.configurable, true);
+        assert.strictEqual(descriptor.enumerable, true);
+        assert.ok(descriptor.get !== undefined);
+        assert.ok(descriptor.set !== undefined);
+      }
+
       const ws = new WebSocket('ws://localhost', { agent: new CustomAgent() });
 
       assert.strictEqual(ws.onmessage, undefined);
@@ -2028,7 +2123,7 @@ describe('WebSocket', () => {
       });
 
       server.listen(0, () => {
-        const ws = new WebSocket(`wss://localhost:${server.address().port}`, {
+        const ws = new WebSocket(`wss://127.0.0.1:${server.address().port}`, {
           rejectUnauthorized: false
         });
       });
@@ -2147,6 +2242,39 @@ describe('WebSocket', () => {
 
       const ws = new WebSocket('wss://127.0.0.1', { servername: '' });
     });
+
+    it("works around a double 'error' event bug in Node.js", function (done) {
+      //
+      // The `minVersion` and `maxVersion` options are not supported in
+      // Node.js < 10.16.0.
+      //
+      if (process.versions.modules < 64) return this.skip();
+
+      //
+      // The `'error'` event can be emitted multiple times by the
+      // `http.ClientRequest` object in Node.js < 13. This test reproduces the
+      // issue in Node.js 12.
+      //
+      const server = https.createServer({
+        cert: fs.readFileSync('test/fixtures/certificate.pem'),
+        key: fs.readFileSync('test/fixtures/key.pem'),
+        minVersion: 'TLSv1.2'
+      });
+      const wss = new WebSocket.Server({ server });
+
+      server.listen(0, () => {
+        const ws = new WebSocket(`wss://localhost:${server.address().port}`, {
+          maxVersion: 'TLSv1.1',
+          rejectUnauthorized: false
+        });
+
+        ws.on('error', (err) => {
+          assert.ok(err instanceof Error);
+          server.close(done);
+          wss.close();
+        });
+      });
+    });
   });
 
   describe('Request headers', () => {
